<?php

/**
 * Copyright (с) Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2023 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * https://www.cloudlinux.com/legal/
 */

namespace WHMCS\Module\Addon\CloudlinuxAdvantage\Actions;

use Illuminate\Database\Capsule\Manager as Capsule;
use Illuminate\Database\Schema\Blueprint;
use Throwable;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\Suite as SuiteModel;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\Pricing;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\ProductConfigGroup;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\ProductConfigOption;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\ProductConfigOptionSub;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Groups\Factory;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Groups\Group;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Groups\MyImunify;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\SuiteJob;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Suites\Statuses\Status;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Suites\Suite;

class Install
{
    /**
     * @return string
     */
    public function getSuitesTableName()
    {
        return (new SuiteModel())->getTable();
    }

    /**
     * @param Group $group class
     * @return ProductConfigGroup
     */
    private function createProductConfigGroup(Group $group)
    {
        $model = (new ProductConfigGroup())->firstOrCreate([
            'name' => $group->name(),
        ], [
            'description' => $group->description(),
        ]);

        if ($model->description !== $group->description()) {
            $model->update([
                'description' => $group->description(),
            ]);
        }

        return $model;
    }

    /**
     * @param int $gid
     * @param array $suites
     * @return void
     */
    private function createProductOptions(int $gid, array $suites)
    {
        $suiteSort = 1;

        foreach ($suites as $suite) {
            /** @var Suite $suiteInstance */
            $suiteInstance = new $suite();

            $configId = $this->createProductConfigOption(
                $gid,
                $suiteInstance->slug(),
                $suiteInstance->name(),
                $suiteSort,
                $suiteInstance->optionType()
            );

            $statusSort = 1;
            $statuses = $suiteInstance->statuses();
            foreach ($statuses as $status) {
                /** @var Status $statusInstance */
                $statusInstance = new $status();

                $relId = $this->createProductConfigOptionSub(
                    $configId,
                    $statusInstance->slug(),
                    $statusInstance->name(),
                    $statusSort
                );
                $this->createPricing($relId);

                $statusSort++;
            }

            $suiteSort++;
        }
    }

    /**
     * @param int $gid
     * @param string $slug
     * @param string $name
     * @param int $sort
     * @param int $option_type
     * @return int
     */
    private function createProductConfigOption(int $gid, string $slug, string $name, int $sort, int $option_type = 2)
    {
        $data = [
            'optionname' => $name,
            'gid' => $gid,
            'optiontype' => $option_type,
            'qtyminimum' => 0,
            'qtymaximum' => 0,
            'order' => $sort,
            'hidden' => false,
        ];

        $exists = ProductConfigOption::where('optionname', 'like', $slug . '|%')->first();

        if (!is_null($exists)) {
            $exists->fill($data);
            $exists->save();

            $model = $exists;
        } else {
            $model = (new ProductConfigOption())->create($data);
        }

        return $model->id;
    }

    /**
     * @param int $configId
     * @param string $slug
     * @param string $name
     * @param int $sort
     * @return int
     */
    private function createProductConfigOptionSub(int $configId, string $slug, string $name, int $sort)
    {
        $data = [
            'configid' => $configId,
            'optionname' => $name,
            'sortorder' => $sort,
            'hidden' => 0,
        ];

        $exists = ProductConfigOptionSub::where('configid', $configId)
            ->where('optionname', 'like', $slug . '|%')
            ->first();

        if (!is_null($exists)) {
            $exists->fill($data);
            $exists->save();

            $model = $exists;
        } else {
            $model = (new ProductConfigOptionSub())->create($data);
        }

        return $model->id;
    }

    /**
     * @param int $relId
     * @return int
     */
    private function createPricing(int $relId)
    {
        $model = (new Pricing())->firstOrCreate([
            'type' => 'configoptions',
            'relid' => $relId,
        ], [
            'currency' => 1, // USD
            'msetupfee' => 0.00,
            'qsetupfee' => 0.00,
            'ssetupfee' => 0.00,
            'asetupfee' => 0.00,
            'bsetupfee' => 0.00,
            'tsetupfee' => 0.00,
            'monthly' => 0.00,
            'quarterly' => 0.00,
            'semiannually' => 0.00,
            'annually' => 0.00,
            'biennially' => 0.00,
            'triennially' => 0.00,
        ]);

        return $model->id;
    }

    /**
     * @return void
     */
    private function createModuleTables()
    {
        $schema = Capsule::schema();

        if (!$schema->hasTable($this->getSuitesTableName())) {
            $schema->create($this->getSuitesTableName(), function (Blueprint $table) {
                $table->increments('id');
                $table->bigInteger('service_id')->index();
                $table->bigInteger('server_id')->index()->nullable();
                $table->string('suite_slug')->nullable();
                $table->string('suite_status')->nullable();
                $table->string('service_status')->nullable();
                $table->mediumText('report')->nullable();
                $table->timestamps();
            });
        }

        $this->upgradeAfterRename();
    }

    /**
     * @return void
     */
    public function all()
    {
        $factory = new Factory();
        $groups = $factory::GROUPS;
        foreach ($groups as $slug => $group) {
            $instance = $factory->get($slug);

            if (is_null($instance)) {
                continue;
            }

            $model = $this->createProductConfigGroup($instance);

            $this->createProductOptions($model->id, $instance->suites());
        }

        $this->createModuleTables();
        SuiteJob::register();
    }

    /**
     * @return void
     */
    public function drop()
    {
        Capsule::schema()->dropIfExists($this->getSuitesTableName());
        SuiteJob::deregister();
    }

    /**
     * @return void
     */
    private function upgradeAfterRename()
    {
        $schema = Capsule::schema();
        if (
            $schema->hasTable('mod_acceleratewp_suites') and
            $this->getSuitesTableName() !== 'mod_acceleratewp_suites'
        ) {
            $items = [];
            $suites = (new SuiteModel())->setTable('mod_acceleratewp_suites')->get();
            foreach ($suites as $suite) {
                $fields = $suite->toArray();
                unset($fields['id']);
                $items[] = $fields;
            }

            if (!empty($items)) {
                try {
                    Capsule::table($this->getSuitesTableName())->insert($items);
                } catch (Throwable $e) {
                    //
                }
            }
        }
    }

    /**
     * @return void
     */
    public function upgrade003()
    {
        $factory = new Factory();

        $instance = $factory->get(MyImunify::SLUG);

        if (is_null($instance)) {
            return;
        }

        $model = $this->createProductConfigGroup($instance);

        $this->createProductOptions($model->id, $instance->suites());
    }
}
