<?php

/**
 * Copyright (с) Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2023 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * https://www.cloudlinux.com/legal/
 */

namespace WHMCS\Module\Addon\CloudlinuxAdvantage\Services;

use Exception;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Actions\Install;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\Suite as SuiteModel;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\ProductConfigOption;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Suites\Factory as SuitesFactory;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Suites\Statuses\Factory as StatusesFactory;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Suites\Suite;
use WHMCS\Scheduling\Jobs\Queue;

class AdminManager
{
    /**
     * @return array
     */
    public function options()
    {
        $options = [];
        $admin_url = Config::instance()->baseUrlAdmin();
        $addon_url = Config::instance()->baseUrlAdminAddon();
        $items = ProductConfigOption::cloudLinuxNames()->with('productConfigOptionGroup')->get();
        $factory = new SuitesFactory();
        $suites = $factory::SUITES;
        $exists = [];

        foreach ($items as $item) {
            $slug = $item->cloudLinuxSlug();
            $suiteInstance = $factory->get($slug);
            $name = '';
            if (!empty($item->productConfigOptionGroup)) {
                $name .= $item->productConfigOptionGroup->name;
            }
            $name .= strstr($item->optionname, '|');
            $exists[] = $slug;
            $options[] = [
                'id' => $item->id,
                'slug' => $slug,
                'url' => $admin_url . '/configproductoptions.php?action=managegroup&id=' . $item->gid,
                'name' => $name,
                'gid' => $item->gid,
                'pushable' => ($suiteInstance and $suiteInstance->pushable()),
            ];
        }

        foreach ($suites as $suite) {
            /** @var Suite $instance */
            $instance = new $suite();
            $slug = $instance->slug();

            if (in_array($slug, $exists)) {
                continue;
            }

            $options[] = [
                'id' => 0,
                'slug' => $slug,
                'url' => $addon_url . '&action=install',
                'name' => $instance->name(),
                'gid' => 0,
                'pushable' => false,
            ];
        }

        return $options;
    }

    /**
     * @return array
     */
    public function suites()
    {
        $suites = [];
        $addon_url = Config::instance()->baseUrlAdminAddon();
        $items = SuiteModel::with([
            'server',
            'service' => function ($query) {
                return $query->with([
                    'configOptionsCloudLinux' => function ($query) {
                        return $query->with([
                            'productConfigOptionCloudLinux',
                            'productConfigOptionSubCloudLinux',
                        ]);
                    }
                ]);
            }
        ])->latest()->get();

        $token = generate_token('plain');
        $queues = Queue::where('name', 'like', SuiteQueue::QUEUE_PREFIX_NAME . '%')->get();
        /** @var SuiteModel $item */
        foreach ($items as $item) {
            $queue = $queues->where('name', SuiteQueue::QUEUE_PREFIX_NAME . $item->id)->first();

            $suite['service_id'] = $item->service_id;
            $suite['service_title'] = $item->service_id;
            $suite['server_id'] = $item->server_id;
            $suite['server_title'] = $item->server_id;
            $suite['suite_slug'] = $item->suite_slug;
            $suite['suite_status'] = $item->suite_status;
            $suite['service_status'] = !empty($item->service_status) ? $item->service_status : '-';
            $suite['report'] = !empty($item->report) ? htmlspecialchars($item->report) : '';

            $service = $item->service;
            if (!empty($service) && !empty($service->domain)) {
                $suite['service_title'] = $service->domain;
            }

            $server = $item->server;
            if (!empty($server) && !empty($server->name)) {
                $suite['server_title'] = $server->name;
            }

            $suiteInstance = (new SuitesFactory())->get((string)$item->suite_slug);
            $statusInstance = (new StatusesFactory())->get((string)$item->suite_status);

            $suite['suite_slug_print'] = $suiteInstance ? $suiteInstance->print() : '-';
            $suite['suite_status_print'] = $statusInstance ? $statusInstance->print() : '-';
            $suite['queue_available_at'] = !empty($queue) ? $queue->available_at : null;
            $suite['queue_started_at'] = !empty($queue) ? $queue->started_at : null;
            $suite['retry_url'] = $addon_url . '&action=suiteConfigure&id=' . $item->id . '&token=' . $token;
            $suite['delete_url'] = $addon_url . '&action=suiteDelete&id=' . $item->id . '&token=' . $token;

            $this->defineActualSuiteStatus($item, $suite);

            $suites[$item->id] = $suite;
        }

        return $suites;
    }

    /**
     * @param SuiteModel $item
     * @param array $suite
     * @return void
     */
    private function defineActualSuiteStatus(SuiteModel $item, array &$suite)
    {
        $suite['current_service_status'] = null;
        $suite['current_suite_status_print'] = null;

        $service = $item->service;
        if ($service) {
            $suite['current_service_status'] = $service->domainstatus;

            foreach ($service->configOptionsCloudLinux as $configOption) {
                $option = $configOption->productConfigOptionCloudLinux;
                $sub = $configOption->productConfigOptionSubCloudLinux;

                if ($option->cloudLinuxSlug() === $item->suite_slug) {
                    $statusInstance = (new StatusesFactory())->get((string)$sub->cloudLinuxSlug());
                    $suite['current_suite_status_print'] = $statusInstance ? $statusInstance->print() : '-';

                    $actualStatusSlug = (new SuiteManager())->defineActualSuiteStatusSlug(
                        $sub->cloudLinuxSlug(),
                        $service->domainstatus
                    );
                    $actualStatusInstance = (new StatusesFactory())->get($actualStatusSlug);
                    $suite['actual_suite_status_print'] = $actualStatusInstance ? $actualStatusInstance->print() : '-';
                    break;
                }
            }
        }
    }

    /**
     * @return void
     */
    public function install()
    {
        (new Install())->all();
    }

    /**
     * @param int $id
     * @return void
     */
    public function suiteConfigure(int $id)
    {
        (new SuiteManager())->configure($id, true)->send();

        Queue::where('name', SuiteQueue::QUEUE_PREFIX_NAME . $id)->delete();
    }

    /**
     * @param int $id
     * @return void
     */
    public function suiteDelete(int $id)
    {
        (new SuiteManager())->delete($id);
    }

    /**
     * @param int $gid
     * @return string
     * @throws Exception
     */
    public function pushInfo(int $gid)
    {
        return (new PushData())->collect($gid)->info();
    }

    /**
     * @param int $gid
     * @return void
     */
    public function pushSend(int $gid)
    {
        Queue::add(
            PushQueue::QUEUE_PREFIX_NAME . $gid,
            PushQueue::class,
            'configure',
            array($gid),
            0,
            true
        );
    }

    /**
     * @return string
     */
    public function pushInfoEndpoint()
    {
        $addon_url = Config::instance()->baseUrlAdminAddon();
        $addon_url .= '&action=pushInfo&token=' . generate_token('plain');
        return $addon_url;
    }

    /**
     * @return string
     */
    public function pushSendEndpoint()
    {
        $addon_url = Config::instance()->baseUrlAdminAddon();
        $addon_url .= '&action=pushSend&token=' . generate_token('plain');
        return $addon_url;
    }
}
