<?php

/**
 * Copyright (с) Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2023 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * https://www.cloudlinux.com/legal/
 */

namespace WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Api;

use Exception;
use WHMCS\Module\Server;

class Cpanel extends Client
{
    /**
     * @var string
     */
    private $cookies = '';

    /**
     * @var string
     */
    private $endpoint = '';

    /**
     * @var string[]
     */
    private $headers = [];

    /**
     * @return string
     */
    public function cookies()
    {
        return $this->cookies;
    }

    /**
     * @return string
     */
    public function endpoint()
    {
        return $this->endpoint;
    }

    /**
     * @return string[]
     */
    public function headers()
    {
        return $this->headers;
    }

    /**
     * @param string $key
     * @param string $value
     *
     * @return self
     */
    public function setHeader($key, $value)
    {
        $this->headers[$key] = $key . ': ' . $value;
        return $this;
    }

    /**
     * @return void
     * @throws Exception
     */
    private function setup()
    {
        $moduleInterface = new Server();
        $url = $moduleInterface->getSingleSignOnUrlForAdmin($this->serverId());

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_TIMEOUT, 25);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

        $response = curl_exec($ch);

        if (curl_errno($ch)) {
            throw new Exception('Error when connecting to the cPanel ' . curl_error($ch));
        }

        curl_close($ch);

        if (is_string($response)) {
            $this->setCookie($response);
            $this->setEndpoint($response, $url);
        }
    }

    /**
     * @param string $response
     * @return void
     */
    public function setCookie(string $response)
    {
        $this->cookies = '';

        preg_match_all('/^Set-Cookie:\s*([^;]*)/im', $response, $matches);

        if (!array_key_exists(1, $matches)) {
            return;
        }

        $this->cookies = implode('; ', $matches[1]);
    }

    /**
     * @param string $response
     * @param string $url
     * @return void
     */
    public function setEndpoint(string $response, string $url)
    {
        $this->endpoint = '';

        preg_match_all('/^Location:\s*([^;]*)/mi', $response, $matches);

        if (!array_key_exists(1, $matches) || !array_key_exists(0, $matches[1])) {
            return;
        }

        $cpsess = '';
        $location = $matches[1][0];

        $matches = [];
        preg_match('/cpsess(\d+)\//', $location, $matches);
        if (!empty($matches)) {
            $cpsess = $matches[1];
        }

        $parts = explode('/cpsess', $url);
        $endpoint = $parts[0];

        $this->endpoint = $endpoint . '/cpsess' . $cpsess;
    }

    /**
     * @param string|array $body
     * @param string $endpoint
     * @param string $method
     * @return array
     * @throws Exception
     */
    public function request($body, string $endpoint, string $method = 'GET')
    {
        if (empty($this->endpoint())) {
            $this->setup();
        }

        $ch = curl_init();

        $url = $this->endpoint() . $endpoint;

        if ($method === 'GET' && is_array($body)) {
            $url .= '?' . http_build_query($body);
        } elseif ($method === 'POST') {
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $body);
        }

        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_TIMEOUT, $this->timeout());
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_COOKIE, $this->cookies());
        curl_setopt($ch, CURLOPT_HTTPHEADER, $this->headers());
        $response = curl_exec($ch);
        $error = '';
        if (curl_errno($ch)) {
            $error = curl_error($ch);
        }
        $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if ($code !== 200) {
            throw new Exception(trim('Wrong HTTP code: ' . $code . ' ' . $error));
        }

        if (empty($response) || !is_string($response)) {
            throw new Exception('Empty response');
        }

        $data = json_decode($response, true);
        if (!is_array($data)) {
            throw new Exception('Unexpected response: ' . $response);
        }

        if (!array_key_exists('result', $data)) {
            throw new Exception('Unexpected response: ' . $response);
        }

        return $data;
    }
}
