<?php

/**
 * Copyright (с) Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2023 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * https://www.cloudlinux.com/legal/
 */

namespace WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Api;

use Exception;
use WHMCS\Module\Server;

class Plesk extends Client
{
    /**
     * @var string
     */
    private $endpoint = '';

    /**
     * @var array
     */
    private $headers = [];

    /**
     * @return string
     */
    public function endpoint()
    {
        return $this->endpoint;
    }

    /**
     * @return array
     */
    public function headers()
    {
        return $this->headers;
    }

    /**
     * @return array
     */
    protected function serverParams()
    {
        return (new Server())->getServerParams($this->serverId());
    }

    /**
     * @return void
     * @throws Exception
     */
    public function setup()
    {
        $params = $this->serverParams();
        $this->setEndpoint($params);
        $this->setHeaders($params);
    }

    /**
     * @param array $params
     * @return void
     */
    public function setEndpoint(array $params)
    {
        $protocol = array_key_exists('serverhttpprefix', $params) ? $params['serverhttpprefix'] : '';
        $host = array_key_exists('serverhostname', $params) ? $params['serverhostname'] : '';
        $port = array_key_exists('serverport', $params) ? ':' . $params['serverport'] : '';
        $this->endpoint = $protocol . '://' . $host . $port . '/enterprise/control/agent.php';
    }

    /**
     * @param array $params
     * @return void
     */
    public function setHeaders(array $params)
    {
        $key = array_key_exists('serveraccesshash', $params) ? $params['serveraccesshash'] : '';
        $login = array_key_exists('serverusername', $params) ? $params['serverusername'] : '';
        $password = array_key_exists('serverpassword', $params) ? $params['serverpassword'] : '';

        $headers = [
            'Content-Type: text/xml',
            'HTTP_PRETTY_PRINT: TRUE',
        ];

        if (!empty($key)) {
            $headers[] = 'KEY: ' . $key;
        } else {
            $headers[] = 'HTTP_AUTH_LOGIN: ' . $login;
            $headers[] = 'HTTP_AUTH_PASSWD: ' . $password;
        }

        $this->headers = $headers;
    }

    /**
     * @param string $body
     * @param string $xpath
     * @return array
     * @throws Exception
     */
    public function request(string $body, string $xpath = 'extension/call/result/accelerate-wp/cloudlinux-awp-admin')
    {
        if (empty($this->endpoint())) {
            $this->setup();
        }

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $this->endpoint());
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $body);
        curl_setopt($ch, CURLOPT_TIMEOUT, $this->timeout());
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $this->headers());
        $response = curl_exec($ch);
        $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if (is_bool($response)) {
            $response = '';
        }

        $result = $this->parse($xpath, $response);

        if ($code !== 200 && $result['code'] == 0) {
            throw new Exception('Wrong HTTP code: ' . $code);
        }

        $data = json_decode($result['stdout'], true);
        if (!is_array($data)) {
            throw new Exception('Unexpected response: ' . $response);
        }

        if (array_key_exists('result', $data) && $data['result'] === 'success') {
            return $data;
        } else {
            throw new Exception('Unexpected response: ' . $response);
        }
    }

    /**
     * @return bool
     */
    public function simplexmlLoaded()
    {
        return extension_loaded('simplexml');
    }

    /**
     * @param ?string $response
     * @param string $xpath
     * @return array
     */
    public function parse(string $xpath = '', $response = null)
    {
        $result = [
            'code' => 0,
            'stdout' => '',
            'stderr' => '',
        ];

        if (empty($response)) {
            return $result;
        }

        if ($this->simplexmlLoaded()) {
            $xml = simplexml_load_string($response);
            if ($xml !== false) {
                $path = $xml->xpath($xpath);
                if (!empty($path)) {
                    $awp = $path[0];
                    $result['code'] = (int)$awp->code;
                    $result['stdout'] = (string)$awp->stdout;
                    $result['stderr'] = (string)$awp->stderr;
                }
            }
        } else {
            preg_match('#<code>(.*?)</code>#is', $response, $code);
            if (array_key_exists(1, $code)) {
                $result['code'] = $code[1];
            }
            preg_match('#<stdout>(.*?)</stdout>#is', $response, $stdout);
            if (array_key_exists(1, $stdout)) {
                $result['stdout'] = $stdout[1];
            }
            preg_match('#<stderr>(.*?)</stderr>#is', $response, $stderr);
            if (array_key_exists(1, $stderr)) {
                $result['stdout'] = $stderr[1];
            }
        }

        return $result;
    }
}
