<?php

/**
 * Copyright (с) Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2023 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * https://www.cloudlinux.com/legal/
 */

namespace WHMCS\Module\Addon\CloudlinuxAdvantage\Services;

use WHMCS\Module\Addon\CloudlinuxAdvantage\Controllers\ClientController;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\Suite as SuiteModel;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\OrderStatus;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\ProductConfigOption;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\Service;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Suites\Factory as SuitesFactory;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Suites\Statuses\Factory as StatusesFactory;
use WHMCS\Scheduling\Jobs\Queue;
use WHMCS\Service\Upgrade\Upgrade;
use WHMCS\Order\Order;

class HookManager
{
    /**
     * Default hook priority
     */
    public const DEFAULT_PRIORITY = 999;

    /**
     * @return void
     */
    public function init()
    {
        $this->order();
        $this->module();
        $this->clientArea();
        $this->serviceEdit();
        $this->afterConfigOptionsUpgrade();
        $this->afterShoppingCartCheckout();
    }

    /**
     * @note Order changed status
     * @return void
     */
    private function order()
    {
        $hooks = [
            'AcceptOrder',
            'CancelOrder',
            'CancelAndRefundOrder',
            'FraudOrder',
            'PendingOrder',
            'DeleteOrder',
        ];

        $statuses = OrderStatus::all();
        foreach ($statuses as $status) {
            $slug = ucfirst($status->title);
            $hooks[] = $slug . 'Order';
            $hooks[] = 'CloudLinux' . $slug . 'Order';
        }

        foreach ($hooks as $hook) {
            add_hook($hook, self::DEFAULT_PRIORITY, function ($vars) {
                if (!is_array($vars)) {
                    $vars = [];
                }

                $id = (int)$vars['orderid'];
                $this->manageStatusByOrder($id);
            });
        }
    }

    /**
     * @note Order changed status
     * @return void
     */
    private function module()
    {
        $hooks = [
            'AfterModuleCreate',
            'AfterModuleCreateFailed',
            'AfterModuleSuspend',
            'AfterModuleSuspendFailed',
            'AfterModuleUnsuspend',
            'AfterModuleUnsuspendFailed',
            'AfterModuleTerminate',
            'AfterModuleTerminateFailed',
            'CancellationRequest',
        ];

        foreach ($hooks as $hook) {
            add_hook($hook, self::DEFAULT_PRIORITY, function ($vars) {
                if (!is_array($vars)) {
                    $vars = [];
                }

                $id = 0;
                if (array_key_exists('params', $vars) && array_key_exists('serviceid', $vars['params'])) {
                    $id = (int)$vars['params']['serviceid'];
                }
                if (array_key_exists('serviceid', $vars)) {
                    $id = (int)$vars['serviceid'];
                } elseif (array_key_exists('relid', $vars)) {
                    $id = (int)$vars['relid'];
                }

                if ($id > 0) {
                    $this->manageStatusByService($id);
                }
            });
        }
    }

    /**
     * @return void
     */
    private function afterConfigOptionsUpgrade()
    {
        add_hook('AfterConfigOptionsUpgrade', self::DEFAULT_PRIORITY, function ($vars) {
            if (!is_array($vars)) {
                $vars = [];
            }

            $id = (int)$vars['upgradeid'];

            /** @var ?Upgrade $upgrade */
            $upgrade = Upgrade::find($id);

            if (empty($upgrade)) {
                Debug::log('HookManager', 'AfterConfigOptionsUpgrade', 'Upgrade model not found');
                return;
            }

            if ($upgrade->type !== Upgrade::TYPE_CONFIGOPTIONS) {
                Debug::log('HookManager', 'AfterConfigOptionsUpgrade', 'Upgrade model type is not configurable option');
                return;
            }

            $service = $upgrade->service;
            if (empty($service)) {
                Debug::log('HookManager', 'AfterConfigOptionsUpgrade', 'Upgrade model service not found');
                return;
            }

            $this->manageStatusByService($service->id);
        });
    }

    /**
     * @return void
     */
    private function afterShoppingCartCheckout()
    {
        add_hook('AfterShoppingCartCheckout', self::DEFAULT_PRIORITY, function ($vars) {
            if (!is_array($vars)) {
                $vars = [];
            }

            $id = (int)$vars['OrderID'];
            $this->manageStatusByOrder($id);
        });
    }

    /**
     * @return void
     */
    private function serviceEdit()
    {
        add_hook('ServiceEdit', self::DEFAULT_PRIORITY, function ($vars) {
            if (!is_array($vars)) {
                $vars = [];
            }

            $id = (int)$vars['serviceid'];
            $this->manageStatusByService($id);
        });
    }

    /**
     * @param string $output
     * @return void
     */
    protected function clientAreaRegisterShutdown($output)
    {
        register_shutdown_function(function () use ($output) {
            echo $output;
        });
    }

    /**
     * @return void
     */
    private function clientArea()
    {
        add_hook('ClientAreaFooterOutput', self::DEFAULT_PRIORITY, function (array $vars) {
            $js = '<script>';
            $js .= 'if(window.opener && !window.opener.closed) {';
            $js .= 'window.opener.postMessage(\'PAYMENT_SUCCESS\', \'*\');';
            $js .= '}';
            $js .= '</script>';

            $isPaid = array_key_exists('ispaid', $vars) ? $vars['ispaid'] : false;
            $paymentSuccess = array_key_exists('paymentSuccess', $vars) ? $vars['paymentSuccess'] : false;
            $templateFile = array_key_exists('templatefile', $vars) ? $vars['templatefile'] : '';
            $invoiceId = array_key_exists('invoiceid', $vars) ? (int)$vars['invoiceid'] : 0;

            if ($templateFile === 'viewinvoice' && $paymentSuccess === true) {
                $this->clientAreaRegisterShutdown($js);
            } elseif ($isPaid || ($templateFile === 'complete' and $invoiceId === 0)) {
                return $js;
            }

            return '';
        });

        add_hook('ClientAreaFooterOutput', self::DEFAULT_PRIORITY, function (array $vars) {
            $type = array_key_exists('type', $vars) ? $vars['type'] : false;
            $templateFile = array_key_exists('templatefile', $vars) ? $vars['templatefile'] : '';
            $configOptions = array_key_exists('configoptions', $vars) ? $vars['configoptions'] : [];

            // Passed with upgrade_url
            $suite = array_key_exists('suite', $_GET) ? $_GET['suite'] : '';

            if ($templateFile !== 'upgrade' || $type !== 'configoptions' || empty($configOptions) || empty($suite)) {
                return '';
            }

            $js = '';

            $configOptionCloudLinux = ProductConfigOption::cloudLinuxNames()->select('id', 'optionname')->get();
            $optionCloudLinuxIds = [];
            foreach ($configOptionCloudLinux as $configOption) {
                $optionCloudLinuxIds[$configOption->id] = $configOption->cloudLinuxSlug();
            }

            foreach ($configOptions as $configOption) {
                if (!array_key_exists($configOption['id'], $optionCloudLinuxIds)) {
                    continue;
                }

                $suiteSlug = $optionCloudLinuxIds[$configOption['id']];
                $suiteInstance = (new SuitesFactory())->get((string)$suiteSlug);

                if (is_null($suiteInstance)) {
                    continue;
                }

                $selector = '[name="configoption[' . $configOption['id'] . ']"]';

                // Display only current
                if (in_array($suite, $suiteInstance->aliases())) {
                    $js .= PHP_EOL . '<script>';
                    $js .= 'var cloudLinuxSelectSuite = document.querySelector(\'' . $selector . '\');';
                    $js .= 'if (cloudLinuxSelectSuite) {';
                    $js .= 'var cloudLinuxSelectSuiteTr = cloudLinuxSelectSuite.closest(\'tr\');';
                    $js .= 'if (cloudLinuxSelectSuiteTr) {';
                    $js .= 'var cloudLinuxSelectSuiteTBody = cloudLinuxSelectSuite.closest(\'tbody\');';
                    $js .= 'if (cloudLinuxSelectSuiteTBody) {';
                    $js .= 'var cloudLinuxSelectSuiteTBodyTr = cloudLinuxSelectSuiteTBody.querySelectorAll(\'tr\');';
                    $js .= 'for (var i = 0; i < cloudLinuxSelectSuiteTBodyTr.length; i++) {';
                    $js .= 'cloudLinuxSelectSuiteTBodyTr[i].style.display = \'none\';';
                    $js .= '}';
                    $js .= 'cloudLinuxSelectSuiteTr.style.display = \'table-row\';';
                    $js .= '}';
                    $js .= '}';
                    $js .= '}';
                    $js .= '</script>';

                    $options = [];
                    foreach ($configOption['options'] as $option) {
                        $rawParts = explode('|', $option['rawName']);
                        $statusSlug = (string)$rawParts[0];
                        $statusInstance = (new StatusesFactory())->get($statusSlug);
                        $selected = (array_key_exists('selected', $option) && $option['selected'] === true);
                        if (is_null($statusInstance)) {
                            continue;
                        }

                        $options[$statusInstance->slug()] = [
                            'id' => $option['id'],
                            'selected' => $selected,
                        ];
                    }

                    $preselectSlug = $suiteInstance->preselectStatusWhenUpgradeSlug($options);
                    if (!empty($preselectSlug)) {
                        $js .= PHP_EOL . '<script>';
                        $js .= 'var cloudLinuxPreSelectSuite = document.querySelector(\'' . $selector . '\');';
                        $js .= 'if (cloudLinuxPreSelectSuite) {';
                        $js .= 'cloudLinuxPreSelectSuite.value = ' . $options[$preselectSlug]['id'] . ';';
                        $js .= '}';
                        $js .= '</script>';
                        break;
                    }
                }
            }

            return $js;
        });

        add_hook('ClientAreaPage', self::DEFAULT_PRIORITY, function (array $vars) {
            if (is_array($_GET)) {
                $module = array_key_exists('m', $_GET) ? $_GET['m'] : '';
                $action = array_key_exists('action', $_GET) ? $_GET['action'] : '';
                if (in_array($module, ['acceleratewp', 'cloudlinux_advantage'])) {
                    $this->clientAreaPageController()->dispatch($action, $vars);
                }
            }

            return $vars;
        });

        // Support for the name of the old version of the module if the old module is still installed
        register_shutdown_function(function () {
            if (is_array($_GET) && array_key_exists('m', $_GET) && $_GET['m'] === 'acceleratewp') {
                run_hook('ClientAreaPage', []);
            }
        });
    }

    /**
     * @return ClientController
     */
    protected function clientAreaPageController()
    {
        $manager = new ClientManager();
        return (new ClientController($manager));
    }

    /**
     * @param int $id
     * @return void
     */
    private function manageStatusByOrder(int $id)
    {
        Debug::log('HookManager', 'ManageStatusByOrder', 'Order id: ' . $id);

        /** @var ?Order $order */
        $order = Order::find($id);

        if (empty($order)) {
            Debug::log('HookManager', 'ManageStatusByOrder', 'Order not found');
            return;
        }

        $services = $order->services;

        foreach ($services as $service) {
            $this->manageStatusByService($service->id);
        }
    }

    /**
     * @param int $id
     * @return void
     */
    private function manageStatusByService(int $id)
    {
        Debug::log('HookManager', 'ManageStatusByService', 'Service id: ' . $id);

        /** @var ?Service $service */
        $service = Service::find($id);

        if (empty($service)) {
            Debug::log('HookManager', 'ManageStatusByService', 'Service not found');
            return;
        }

        $server = $service->serverModel;
        $options = $service->configOptionsCloudLinux;
        foreach ($options as $option) {
            $optionCloudLinux = $option->productConfigOptionCloudLinux;
            $suite = SuiteModel::firstOrCreate([
                'service_id' => $service->id,
                'server_id' => $server ? $server->id : null,
                'suite_slug' => $optionCloudLinux ? $optionCloudLinux->cloudLinuxSlug() : null,
            ], [
                'service_status' => null,
                'suite_status' => null,
            ]);

            $suite->report = 'In the process of updating';
            $suite->save();

            $suiteId = $suite->id;
            $this->addQueue($suiteId);
            Debug::log('HookManager', 'ManageStatusByService', 'Suite ' . $suiteId . ' queue created');
        }
    }

    /**
     * @param int $suiteId
     * @return void
     */
    protected function addQueue(int $suiteId)
    {
        Queue::add(
            SuiteQueue::QUEUE_PREFIX_NAME . $suiteId,
            SuiteQueue::class,
            'configure',
            array($suiteId),
            0,
            true
        );
    }
}
