<?php

/**
 * Copyright (с) Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2023 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * https://www.cloudlinux.com/legal/
 */

namespace WHMCS\Module\Addon\CloudlinuxAdvantage\Services;

use Exception;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\Product;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\Service;
use WHMCS\Product\Server;

class PushData
{
    /**
     * @var string[]
     */
    protected $default = [
        'type' => 'default',
        'group_id' => '',
        'option_id' => '',
        'option_name_print' => '',
        'option_cloudlinux_slug' => '',
        'option_sub_id' => '',
        'option_sub_name_print' => '',
        'option_sub_cloudlinux_slug' => '',
    ];

    /**
     * @var array
     */
    protected $data = [];

    /**
     * @return array
     */
    protected function default()
    {
        return $this->default;
    }

    /**
     * @return array
     */
    public function data()
    {
        return $this->data;
    }

    /**
     * @param Product $product
     * @return void
     * @throws Exception
     */
    protected function checkGroupCount(Product $product)
    {
        $duplicates = [];
        // It's important get fresh data
        $groups = $product->productConfigGroups()->with('productConfigOptions')->get();
        foreach ($groups as $group) {
            foreach ($group->productConfigOptions as $option) {
                $slug = $option->cloudLinuxSlug();

                if (!array_key_exists($slug, $duplicates)) {
                    $duplicates[$option->cloudLinuxSlug()] = 0;
                }

                $duplicates[$slug]++;
            }
        }

        foreach ($duplicates as $key => $count) {
            if ($count > 1) {
                throw new Exception('The "' . $product->name . '" product contains more than'
                    . 'one linked "' . $key . '".');
            }
        }
    }

    /**
     * @param Product $product
     * @return void
     */
    protected function setProduct(Product $product)
    {
        if (!array_key_exists($product->id, $this->data)) {
            $this->data[$product->id] = [
                'product_id' => $product->id,
                'product_name' => $product->name,
                'servers' => [],
            ];
        }
    }

    /**
     * @param int $product_id
     * @param Server $server
     * @return void
     */
    protected function setServer(int $product_id, Server $server)
    {
        if (!array_key_exists($server->id, $this->data[$product_id]['servers'])) {
            $this->data[$product_id]['servers'][$server->id] = [
                'server_name' => $server->name,
                'server_ip' => $server->ipaddress,
                'server_id' => $server->id,
                'options' => [],
            ];
        }
    }

    /**
     * @param Product $product
     * @return array
     */
    protected function fillDefaultOption(Product $product)
    {
        $default = $this->default();
        foreach ($product->productConfigGroups as $group) {
            $default['group_id'] = $group->id;
            foreach ($group->productConfigOptions as $option) {
                $default['option_id'] = $option->id;
                $default['option_name_print'] = $this->namePrint($option->optionname);
                $default['option_cloudlinux_slug'] = $option->cloudLinuxSlug();

                foreach ($option->productConfigOptionSubs->sortBy('sortorder') as $sub) {
                    $default['option_sub_id'] = $sub->id;
                    $default['option_sub_name_print'] = $this->namePrint($sub->optionname);
                    $default['option_sub_cloudlinux_slug'] = $sub->cloudLinuxSlug();

                    break; // First by sort order is default
                }
            }
        }

        return $default;
    }

    /**
     * @param Service $service
     * @param array $defaultOption
     * @return array
     */
    protected function fillUserOption(Service $service, array $defaultOption)
    {
        $option = $defaultOption;
        if ($service->configOptionsCloudLinux->count()) {
            foreach ($service->configOptionsCloudLinux as $service_rel) {
                if (empty($service_rel)) {
                    continue;
                }

                $service_option = $service_rel->productConfigOptionCloudLinux;
                if (empty($service_option)) {
                    continue;
                }

                $service_group = $service_option->productConfigOptionGroup;
                $service_sub = $service_rel->productConfigOptionSubCloudLinux;
                if (empty($service_sub)) {
                    continue;
                }

                if ($service_group->id == $option['group_id']) {
                    $option['type'] = 'user';
                    $option['option_sub_id'] = $service_sub->id;
                    $option['option_sub_name_print'] = $this->namePrint($service_sub->optionname);
                    $option['option_sub_cloudlinux_slug'] = $service_sub->cloudLinuxSlug();
                    return $option;
                }
            }
        }

        return $option;
    }

    /**
     * @param int $product_id
     * @param int $server_id
     * @param array $option
     * @return void
     */
    protected function setOptionToServer(int $product_id, int $server_id, array $option)
    {
        if (!isset($this->data[$product_id]['servers'][$server_id]['options'][$option['option_sub_id']])) {
            $this->data[$product_id]['servers'][$server_id]['options'][$option['option_sub_id']] = $option;
            $this->data[$product_id]['servers'][$server_id]['options'][$option['option_sub_id']]['users'] = [];
        }
    }

    /**
     * @param int $product_id
     * @param int $server_id
     * @param array $option
     * @param string $user
     * @param string $domain
     * @return void
     */
    protected function setUserToServer(int $product_id, int $server_id, array $option, string $user, string $domain)
    {
        if (
            !array_key_exists(
                $user,
                $this->data[$product_id]['servers'][$server_id]['options'][$option['option_sub_id']]['users']
            )
        ) {
            $this->data[$product_id]['servers'][$server_id]['options'][$option['option_sub_id']]
            ['users'][$user] = $domain;
        }
    }

    /**
     * Collect push data only for MyImunify Account protection.
     *
     * @param int $group_id
     * @return PushData
     * @throws Exception
     */
    public function collect(int $group_id)
    {
        Product::pushDataMyImunifyAccountProtection($group_id)->chunk(100, function ($products) {
            foreach ($products as $product) {
                $this->checkGroupCount($product);

                $this->setProduct($product);
                $defaultOption = $this->fillDefaultOption($product);

                foreach ($product->services as $service) {
                    $server = $service->serverModel;
                    $this->setServer($product->id, $server);
                    $actualOption = $this->fillUserOption($service, $defaultOption);

                    $this->setOptionToServer($product->id, $server->id, $actualOption);
                    $this->setUserToServer(
                        $product->id,
                        $server->id,
                        $actualOption,
                        $service->username,
                        $service->domain ?? ''
                    );
                }
            }
        });

        return $this;
    }

    /**
     * @return string
     */
    public function info()
    {
        $info = [];
        foreach ($this->data() as $product) {
            $string = 'For hosting plan "' . $product['product_name'] . '"';
            $string .= ' changes will be made to the server:<br><br>';
            foreach ($product['servers'] as $server) {
                $string .= '<strong>';
                $string .= 'Server "' . $server['server_name'] . '" (' . $server['server_ip'] . ')';
                $string .= '</strong>:<br>';
                foreach ($server['options'] as $option) {
                    $string .= $option['option_name_print'] . ': ' . $option['option_sub_name_print'] . ' for users: ';
                    $string .= implode(', ', array_keys($option['users'])) . '<br>';
                }
            }
            $info[] = $string;
        }

        return implode('<hr>', $info);
    }

    /**
     * @param ?string $name
     * @return string
     */
    protected function namePrint($name = null)
    {
        $print = '';
        if (is_string($name)) {
            $str = strstr($name, '|');
            if (is_string($str)) {
                $print = trim($str, '|');
            }
        }

        return $print;
    }
}
