<?php

/**
 * Copyright (с) Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2023 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * https://www.cloudlinux.com/legal/
 */

namespace WHMCS\Module\Addon\CloudlinuxAdvantage\Services;

use Throwable;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Api\Api;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Api\Cpanel;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Api\Plesk;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Api\DirectAdmin;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Suites\Factory as SuitesFactory;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Suites\Suite;
use WHMCS\Product\Server;

class PushManager
{
    /** @var int $gid */
    private $gid;

    /** @var PushData $dataProvider */
    private $dataProvider;

    public function __construct(int $gid, PushData $dataProvider)
    {
        $this->gid = $gid;
        $this->dataProvider = $dataProvider;
    }

    /**
     * @return int
     */
    public function gid(): int
    {
        return $this->gid;
    }

    /**
     * @return PushData
     */
    public function dataProvider(): PushData
    {
        return $this->dataProvider;
    }

    /**
     * @return array
     */
    private function data()
    {
        try {
            return $this->dataProvider()->collect($this->gid())->data();
        } catch (Throwable $e) {
            logModuleCall('cloudlinux_advantage', 'pushManager', 'send', $e->getMessage());
            return [];
        }
    }

    /**
     * @param array $option
     * @return string
     */
    private function users(array $option)
    {
        if (array_key_exists('users', $option)) {
            return implode(',', array_keys($option['users']));
        }
        return '';
    }

    /**
     * @param array $option
     * @return string
     */
    private function domains(array $option)
    {
        if (array_key_exists('users', $option)) {
            return implode(',', array_values($option['users']));
        }
        return '';
    }

    /**
     * @param string $serverType
     * @param string $apiClass
     * @param int $serverId
     * @param string $users
     * @param string $domains
     * @return Api|null
     */
    protected function api(string $serverType, string $apiClass, int $serverId, string $users, string $domains)
    {
        $api = null;
        if ($serverType === 'cpanel') {
            $client = Cpanel::getInstance($serverId);
            /** @var Api $api */
            $api = new $apiClass($client, $users);
        } elseif ($serverType === 'plesk') {
            $client = Plesk::getInstance($serverId);
            /** @var Api $api */
            $api = new $apiClass($client, $users, $domains);
        } elseif ($serverType === 'directadmin') {
            $client = DirectAdmin::getInstance($serverId);
            /** @var Api $api */
            $api = new $apiClass($client, $users);
        }

        return $api;
    }

    /**
     * @return void
     */
    public function send()
    {
        $data = $this->data();

        foreach ($data as $product) {
            foreach ($product['servers'] as $server) {
                $serverId = $server['server_id'];
                $serverModel = Server::find($serverId);

                if (empty($serverModel)) {
                    logModuleCall(
                        'cloudlinux_advantage',
                        'pushManager',
                        'Hosting plan "' . $product['product_name'] . '"',
                        'Server ID ' . $serverId . ' not found'
                    );
                    continue;
                }

                foreach ($server['options'] as $option) {
                    $slug = $option['option_cloudlinux_slug'] ?? '';

                    /** @var ?Suite $suiteInstance */
                    $suiteInstance = (new SuitesFactory())->get($slug);

                    if (is_null($suiteInstance)) {
                        logModuleCall(
                            'cloudlinux_advantage',
                            'pushManager',
                            'Hosting plan "' . $product['product_name'] . '", server "'
                            . $server['server_name'] . '" (' . $server['server_ip'] . ')',
                            'Suite "' . $slug . '" not found'
                        );
                        continue;
                    }

                    $serverType = strtolower($serverModel->type ?? '');
                    $apiClass = $suiteInstance->apiClass($serverType);

                    if (empty($apiClass)) {
                        logModuleCall(
                            'cloudlinux_advantage',
                            'pushManager',
                            'Hosting plan "' . $product['product_name'] . '", server "'
                            . $server['server_name'] . '" (' . $server['server_ip'] . ')',
                            $suiteInstance->print() . ' does not support ' . $serverType
                        );
                        continue;
                    }

                    $users = $this->users($option);
                    $domains = $this->domains($option);

                    $api = $this->api($serverType, $apiClass, $serverId, $users, $domains);
                    if (is_null($api)) {
                        logModuleCall(
                            'cloudlinux_advantage',
                            'pushManager',
                            'Hosting plan "' . $product['product_name'] . '", server "'
                            . $server['server_name'] . '" (' . $server['server_ip'] . ')',
                            'Api for ' . $serverType . ' not supported'
                        );
                        continue;
                    }

                    $apiStatus = $suiteInstance->apiStatus($option['option_sub_cloudlinux_slug']) ?? '';
                    $apiSlug = $suiteInstance->apiSlug($option['option_sub_cloudlinux_slug']) ?? '';
                    $apiAttributes = $suiteInstance->apiAttributes($option['option_sub_cloudlinux_slug']) ?? [];

                    if (empty($apiStatus) or empty($apiSlug)) {
                        logModuleCall(
                            'cloudlinux_advantage',
                            'pushManager',
                            'Hosting plan "' . $product['product_name'] . '", server "'
                            . $server['server_name'] . '" (' . $server['server_ip'] . ')',
                            'Sub option slug "' . $option['option_sub_cloudlinux_slug'] . '" is not valid'
                        );
                        continue;
                    }

                    try {
                        $api->callStatus($apiStatus, $apiSlug, $apiAttributes);
                        logModuleCall(
                            'cloudlinux_advantage',
                            'pushManager',
                            'Hosting plan "' . $product['product_name'] . '", server "'
                            . $server['server_name'] . '" (' . $server['server_ip'] . ')',
                            'Update "' . $apiSlug . '" set "' . $apiStatus . '" for users: ' . $users
                        );
                    } catch (Throwable $e) {
                        logModuleCall(
                            'cloudlinux_advantage',
                            'pushManager',
                            'Hosting plan "' . $product['product_name'] . '", server  "'
                            . $server['server_name'] . '" (' . $server['server_ip'] . ')',
                            'Api catch error ' . $e->getMessage()
                        );
                    }
                }
            }
        }
    }
}
