<?php

/**
 * Copyright (с) Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2023 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * https://www.cloudlinux.com/legal/
 */

namespace WHMCS\Module\Addon\CloudlinuxAdvantage\Services;

use Exception;
use Illuminate\Database\Eloquent\Relations\HasOne;
use WHMCS\Carbon;
use WHMCS\Scheduling\Jobs\Queue;
use WHMCS\Scheduling\Task\AbstractTask;

class SuiteJob extends AbstractTask
{
    /**
     * @var int
     */
    protected $defaultPriority = PHP_INT_MAX;

    /**
     * @var int
     */
    protected $defaultFrequency = 1;

    /**
     * @var string
     */
    protected $defaultDescription = "Execute queued jobs for CloudLinuxAdvantage.";

    /**
     * @var string
     */
    protected $defaultName = "Run CloudLinuxAdvantage Jobs Queue";

    /**
     * @var string
     */
    protected $systemName = "RunSuiteJob";

    /**
     * @var bool
     */
    protected $skipDailyCron = true;

    /**
     * @var array[]
     */
    protected $outputs = array(
        "executed" => array(
            "defaultValue" => 0,
            "identifier" => "executed",
            "name" => "Jobs Executed"
        )
    );

    /**
     * @var string
     */
    protected $icon = "fas fa-gavel";

    /**
     * @var string
     */
    protected $successCountIdentifier = "jobs.queue";

    /**
     * @var string
     */
    protected $successKeyword = "Executed";

    /**
     * @return $this
     */
    public function __invoke()
    {
        $this->output("executed")->write($this->executeQueuedJobs());
        return $this;
    }

    /**
     * @return int
     */
    public function executeQueuedJobs()
    {
        $executedCount = 0;
        $jobs = Queue::where("available_at", "<=", Carbon::now()->toDateTimeString())
            ->where('name', 'like', 'cloudlinux_advantage%')
            ->get();
        foreach ($jobs as $job) {
            $className = $job->class_name;
            $methodName = $job->method_name;
            $error = "Exception thrown in jobs queue execution" . " (" . $className . "::" . $methodName . ")" . " - ";
            try {
                $job->delete();
                $job->executeJob();
                $executedCount++;
            } catch (Exception $e) {
                logActivity($error . $e->getMessage());

                try {
                    sleep(5);
                    $job->executeJob();
                    $executedCount++;
                } catch (Exception $e) {
                    logActivity($error . $e->getMessage());
                }
            }
        }
        return $executedCount;
    }

    /**
     * @return SuiteJobStatus
     */
    public function getStatus()
    {
        $status = $this->status()->first();
        if (!$status) {
            $status = SuiteJobStatus::firstOrCreate(array("task_id" => $this->id));
        }
        return $status;
    }

    /**
     * @param mixed $status
     * @return mixed
     */
    public function getStatusAttribute($status)
    {
        if (!$status) {
            $status = $this->getStatus();
        }
        return $status;
    }

    /**
     * @return HasOne
     */
    public function status()
    {
        return $this->hasOne(SuiteJobStatus::class, 'task_id');
    }

    /**
     * @return void
     */
    public static function deregister()
    {
        (new self())->where('class_name', self::class)->delete();
    }
}
