<?php

/**
 * Copyright (с) Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2023 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * https://www.cloudlinux.com/legal/
 */

namespace WHMCS\Module\Addon\CloudlinuxAdvantage\Services;

use Exception;
use Throwable;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\Suite as SuiteModel;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Models\Service;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Api\Api;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Api\Cpanel;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Api\Plesk;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Api\DirectAdmin;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Suites\Factory as SuitesFactory;
use WHMCS\Module\Addon\CloudlinuxAdvantage\Services\Suites\Suite;
use WHMCS\Product\Server;
use WHMCS\Scheduling\Jobs\Queue;

class SuiteManager
{
    /**
     * @var SuiteModel
     */
    private $suite;

    /**
     * @var Service
     */
    private $service;

    /**
     * @var Server
     */
    private $server;

    /**
     * @var Api
     */
    private $api;

    /**
     * @var ?string
     */
    private $addonServiceStatus = null;

    /**
     * @var ?string
     */
    private $addonSuiteSlug = null;

    /**
     * @var ?string
     */
    private $addonSuiteStatusSlug = null;

    /**
     * @var ?string
     */
    private $addonSuiteInitialStatusSlug = null;

    /**
     * @var ?Suite
     */
    private $suiteInstance = null;

    /**
     * @var string
     */
    private $whmcsServiceStatus = '';

    /**
     * @var string
     */
    private $whmcsSuiteStatusSlug = '';

    /**
     * @var string
     */
    private $actualSuiteStatusSlug = '';

    /**
     * @var ?string
     */
    private $error = null;

    /**
     * @param int $suiteId
     * @param bool $skipRequestCheckDuplicate
     * @return self
     */
    public function configure(int $suiteId, bool $skipRequestCheckDuplicate = false)
    {
        $this->setSuite($suiteId);

        try {
            $this->setInitialStatus();
            $this->setService();
            $this->setServer();
            $this->setApi();
            $this->setActualSuiteStatusSlug();
            if ($skipRequestCheckDuplicate === false) {
                $this->requestCheckDuplicate();
            }
        } catch (Throwable $e) {
            Debug::log(
                'SuiteManager',
                'initialization',
                'Suite ' . $suiteId . ': ' . $e->getMessage()
            );

            $this->suite()->report = $e->getMessage();
            $this->suite()->save();

            $this->error = $e->getMessage();
        }

        return $this;
    }

    /**
     * @param int $suiteId
     * @return void
     */
    private function setSuite(int $suiteId)
    {
        $suite = SuiteModel::find($suiteId);

        $this->suite = $suite;
        $this->addonServiceStatus = $suite->service_status;
        $this->addonSuiteSlug = $suite->suite_slug;
        $this->addonSuiteStatusSlug = $suite->suite_status;
    }

    /**
     * @param int $suiteId
     * @return void
     */
    public function delete(int $suiteId)
    {
        try {
            SuiteModel::where('id', $suiteId)->delete();
            Queue::where('name', SuiteQueue::QUEUE_PREFIX_NAME . $suiteId)->delete();
        } catch (Throwable $e) {
            //
        }
    }

    /**
     * @return SuiteModel
     */
    public function suite()
    {
        return $this->suite;
    }

    /**
     * @return string
     */
    public function addonServiceStatus()
    {
        return $this->addonServiceStatus ?? '';
    }

    /**
     * @return string
     */
    public function addonSuiteSlug()
    {
        return $this->addonSuiteSlug ?? '';
    }

    /**
     * @return string
     */
    public function addonSuiteStatusSlug()
    {
        return $this->addonSuiteStatusSlug ?? '';
    }

    /**
     * @return string
     */
    public function addonSuiteInitialStatusSlug()
    {
        return $this->addonSuiteInitialStatusSlug ?? '';
    }

    /**
     * @return void
     * @throws Exception
     */
    private function setInitialStatus()
    {
        $suiteInstance = (new SuitesFactory())->get($this->addonSuiteSlug());

        if (is_null($suiteInstance)) {
            throw new Exception('Suite type ' . $this->addonSuiteSlug() . ' not found');
        }

        $this->suiteInstance = $suiteInstance;
        $this->addonSuiteInitialStatusSlug = $suiteInstance->initialStatusSlug();
    }

    /**
     * @return void
     * @throws Exception
     */
    private function setService()
    {
        $service = $this->suite()->service;

        if (empty($service)) {
            throw new Exception('Model Service not found');
        }

        $this->service = $service;

        $this->whmcsServiceStatus = $service->domainstatus;
        $this->whmcsSuiteStatusSlug = $this->addonSuiteInitialStatusSlug();

        foreach ($service->configOptionsCloudLinux as $configOption) {
            $option = $configOption->productConfigOptionCloudLinux;
            $sub = $configOption->productConfigOptionSubCloudLinux;

            if ($option->cloudLinuxSlug() === $this->addonSuiteSlug()) {
                $this->whmcsSuiteStatusSlug = $sub->cloudLinuxSlug();
                break;
            }
        }
    }

    /**
     * @return Service
     */
    public function service()
    {
        return $this->service;
    }

    /**
     * @return ?Suite
     */
    public function suiteInstance()
    {
        return $this->suiteInstance;
    }

    /**
     * @return string
     */
    public function whmcsServiceStatus()
    {
        return $this->whmcsServiceStatus;
    }

    /**
     * @return string
     */
    public function whmcsSuiteStatusSlug()
    {
        return $this->whmcsSuiteStatusSlug;
    }

    /**
     * @return void
     * @throws Exception
     */
    private function setServer()
    {
        $server = $this->suite()->server;

        if (empty($server)) {
            throw new Exception('Server model not found');
        }

        $this->server = $server;
    }

    /**
     * @return Server
     */
    public function server()
    {
        return $this->server;
    }

    /**
     * @return void
     * @throws Exception
     */
    private function setApi()
    {
        /** @var Suite $suiteInstance */
        $suiteInstance = $this->suiteInstance();

        $user = (string)$this->service()->username;
        $serverId = $this->server()->id;
        $serverType = strtolower($this->server()->type ?? '');
        $domain = $this->service()->domain;
        $apiClass = $suiteInstance->apiClass($serverType);

        if (empty($apiClass)) {
            throw new Exception($suiteInstance->print() . ' does not support ' . $serverType);
        }

        if ($serverType === 'cpanel') {
            $client = Cpanel::getInstance($serverId);

            /** @var Api $apiClass */
            $this->api = new $apiClass($client, $user);
        } elseif ($serverType === 'plesk') {
            $client = Plesk::getInstance($serverId);

            /** @var Api $apiClass */
            $this->api = new $apiClass($client, $user, $domain);
        } elseif ($serverType === 'directadmin') {
            $client = DirectAdmin::getInstance($serverId);

            /** @var Api $apiClass */
            $this->api = new $apiClass($client, $user);
        } else {
            throw new Exception('Api for ' . $serverType . ' not supported');
        }
    }

    /**
     * @return Api
     */
    public function api()
    {
        return $this->api;
    }

    /**
     * @return void
     * @throws Exception
     */
    private function requestCheckDuplicate()
    {
        if (
            $this->whmcsServiceStatus() === $this->addonServiceStatus() &&
            $this->whmcsSuiteStatusSlug() === $this->addonSuiteStatusSlug()
        ) {
            throw new Exception('Nothing todo, the same state');
        }

        if (
            strtolower($this->whmcsServiceStatus()) !== 'active' &&
            empty($this->addonServiceStatus()) &&
            empty($this->addonSuiteStatusSlug())
        ) {
            throw new Exception('Nothing to do, service not configured yet');
        }
    }

    /**
     * @return void
     */
    private function setActualSuiteStatusSlug()
    {
        $this->actualSuiteStatusSlug = $this->defineActualSuiteStatusSlug(
            $this->whmcsSuiteStatusSlug(),
            $this->whmcsServiceStatus()
        );
    }

    /**
     * @param string $whmcsSuiteStatusSlug
     * @param string $whmcsServiceStatus
     * @return string
     */
    public function defineActualSuiteStatusSlug(string $whmcsSuiteStatusSlug, string $whmcsServiceStatus)
    {
        $status = $whmcsSuiteStatusSlug;

        if (strtolower($whmcsServiceStatus) !== 'active') {
            $status = $this->addonSuiteInitialStatusSlug();
        }

        return $status;
    }

    /**
     * @return string
     */
    public function actualSuiteStatusSlug()
    {
        return $this->actualSuiteStatusSlug;
    }

    /**
     * @return void
     */
    public function send()
    {
        if (!empty($this->error)) {
            return;
        }

        $suiteInstance = $this->suiteInstance();
        $suiteSlug = $this->addonSuiteSlug();
        $suiteStatusSlug = $this->actualSuiteStatusSlug();

        try {
            if (is_null($suiteInstance)) {
                throw new Exception('Unexpected suite slug: ' . $suiteSlug);
            }

            $apiStatus = $suiteInstance->apiStatus($suiteStatusSlug);
            $apiSlug = $suiteInstance->apiSlug($suiteStatusSlug);
            $apiAttributes = $suiteInstance->apiAttributes($suiteStatusSlug) ?? [];

            if (is_null($apiStatus) || is_null($apiSlug)) {
                throw new Exception('Unexpected suite status: ' . $suiteStatusSlug);
            }

            $this->api()->callStatus($apiStatus, $apiSlug, $apiAttributes);

            $this->suite()->report = 'Successfully updated';
            $this->suite()->suite_status = $suiteStatusSlug;
            $this->suite()->service_status = $this->whmcsServiceStatus();
            $this->suite()->save();

            $debug = 'Suite ' . $this->suite()->id . ' set ' . $apiStatus . ' status for ' . $suiteSlug;
        } catch (Throwable $e) {
            $debug = 'Api catch error ' . $e->getMessage();

            $this->suite()->report = $debug;
            $this->suite()->save();

            $extra = [
                'suite_slug' => (string)$this->suite()->suite_slug,
                'suite_status' => (string)$this->suite()->suite_status,
                'suite_server_id' => (string)$this->suite()->server_id,

                'service_username' => (string)$this->service()->username,
                'service_domain' => (string)$this->service()->domain,
                'service_status' => (string)$this->service()->domainstatus,
                'service_server_id' => (string)$this->service()->server,

                'server_ip' => (string)$this->server()->ipaddress,
                'server_hostname' => (string)$this->server()->hostname,
                'server_type' => (string)$this->server()->type,
                'server_has_username' => !empty($this->server()->username) ? true : false,
                'server_has_password' => !empty($this->server()->password) ? true : false,
                'server_has_accesshash' => !empty($this->server()->accesshash) ? true : false,
            ];

            Debug::sentry($e->getFile(), $e->getLine(), $debug, 'Info', $extra);
        }

        Debug::log('SuiteManager', 'send', $debug);
    }
}
